import { join } from "path";
import { Configuration, Inject } from "@tsed/di";
import { PlatformApplication } from "@tsed/common";
import "@tsed/platform-express"; // /!\ keep this import
import "@tsed/ajv";
import "@tsed/swagger";
import "@tsed/passport";
import { config } from "./config/index";
import * as rest from "./controllers/rest/index";
import * as pages from "./controllers/pages/index";
import { User } from "./entities/user/User";
import "./protocols/GthubProtocol";

@Configuration({
  ...config,
  acceptMimes: ["application/json"],
  httpPort: process.env.PORT || "127.0.0.1:8081",
  httpsPort: false, // CHANGE
  disableComponentsScan: false,
  ajv: {
    returnsCoercedValues: true
  },
  mount: {
    "/rest": [...Object.values(rest)],
    "/": [...Object.values(pages)]
  },
  swagger: [
    {
      path: "/doc",
      specVersion: "3.0.1"
    }
  ],
  statics: {
    "/static/victormono": {
      root: join(__dirname, "../node_modules/victormono/dist"),
    },
    "/static/missing": {
      root: join(__dirname, "../node_modules/missing.css/dist")
    },
    "/static": {
      root: join(__dirname, "../static")
    }
  },
  componentsScan: [`./protocols/*.ts`, `./services/*.ts`],
  passport: {
    userInfoModel: User
  },
  middlewares: [
    "cors",
    "cookie-parser",
    "compression",
    "method-override",
    "json-parser",
    { use: "urlencoded-parser", options: { extended: true } },
    {
      use: "express-session",
      options: {
        secret: process.env.PASSPORT_SESSION_KEY || "mysecretkey",
        resave: true,
        saveUninitialized: true,
        // maxAge: 36000,
        cookie: {
          path: "/",
          httpOnly: true,
          secure: false
          // maxAge: null
        }
      }
    }
  ],
  views: {
    root: join(process.cwd(), "../views"),
    extensions: {
      ejs: "ejs"
    }
  },
  exclude: ["**/*.spec.ts"]
})
export class Server {
  @Inject()
  protected app: PlatformApplication;

  @Configuration()
  protected settings: Configuration;
}
