import { expect, describe, it, afterAll, beforeAll } from "vitest";
import { PlatformTest } from "@tsed/common";
import SuperTest from "supertest";
import { LinkController } from "./LinkController";
import { Server } from "../../Server";
import { randomUUID } from "crypto";

describe("LinkController", () => {
  beforeAll(
    PlatformTest.bootstrap(Server, {
      mount: {
        "/rest": [LinkController]
      }
    })
  );
  afterAll(PlatformTest.reset);

  it("should call POST /rest/links and GET /rest/links/:id", async () => {
    const request = SuperTest(PlatformTest.callback());
    const username = `silentsilas-${randomUUID()}`;
    const postResponse = await request
      .post("/rest/links")
      .send({
        service: "github",
        serviceIdentifier: username,
        text: "some text",
        iv: "some iv"
      })
      .expect(201);

    const getOneResponse = await request.get(`/rest/users/${postResponse.body.userId}`).expect(200);
    const getAllResponse = await request.get("/rest/users/").expect(200);

    expect(postResponse.body.id).toBeTruthy();
    expect(postResponse.body.service).toEqual("github");
    expect(postResponse.body.serviceIdentifier).toEqual(username);

    expect(getOneResponse.body.id).toEqual(postResponse.body.userId);
    expect(getOneResponse.body.service).toEqual("github");
    expect(getOneResponse.body.serviceIdentifier).toEqual(username);

    expect(getAllResponse.body).toBeInstanceOf(Array);
    expect(getAllResponse.body.length).toBeGreaterThan(0);
  });
});
