import { PathParams } from "@tsed/platform-params";
import { Description, Get, Post, Returns, Summary } from "@tsed/schema";
import { Controller, Inject } from "@tsed/di";
import { User } from "../../entities/user/User";
import { Forbidden } from "@tsed/exceptions";
import { UserService } from "../../services/UserService";

@Controller("/users")
export class UserController {
  @Inject()
  service: UserService;

  // disable the create method and endpoint
  @Post("/")
  @Summary("Create a new user")
  @Description("This endpoint can not be used. Users are created automatically when a new link is created.")
  @Returns(403)
  async create(): Promise<void> {
    throw new Forbidden("Users are created automatically when a new link is created.");
  }

  @Get("/")
  @Summary("Get all users")
  @(Returns(200, Array).Of(User))
  async getList(): Promise<User[]> {
    return this.service.getAllUsers();
  }

  @Get("/:id")
  @Summary("Get a user by ID")
  @Returns(200, User)
  async getOne(@PathParams("id") id: string): Promise<User | null> {
    return this.service.getUserById(id);
  }
}
