import { registerProvider } from "@tsed/di";
import { DataSource } from "typeorm";
import { Logger } from "@tsed/logger";
import { User } from "../entities/user/User";
import { Link } from "../entities/link/Link";

export const SqliteDatasource = Symbol.for("SqliteDatasource");
export type SqliteDatasource = DataSource;
export const sqliteDatasource =
  process.env.NODE_ENV === "test"
    ? new DataSource({
        type: "sqlite",
        entities: [User, Link],
        database: ":memory:",
        synchronize: true,
        extra: {
          connectionLimit: 1,
          busyTimeout: 4000,
          busyErrorRetry: 10
        }
      })
    : new DataSource({
        type: "sqlite",
        entities: [User, Link],
        database: "database.sqlite",
        synchronize: true,
        extra: {
          connectionLimit: 1,
          busyTimeout: 4000,
          busyErrorRetry: 10
        }
      });

registerProvider<DataSource>({
  provide: SqliteDatasource,
  type: "typeorm:datasource",
  deps: [Logger],
  async useAsyncFactory(logger: Logger) {
    if (!sqliteDatasource.isInitialized) {
      await sqliteDatasource.initialize();
      logger.info("Connected with TypeORM to database: Sqlite");
    }
    return sqliteDatasource;
  },
  hooks: {
    async $onDestroy(dataSource: DataSource) {
      if (dataSource.isInitialized) {
        await dataSource.destroy();
      }
    }
  }
});
