import { Inject, Injectable } from "@tsed/di";
import { Forbidden, NotFound } from "@tsed/exceptions";
import { SqliteDatasource } from "../datasources/SqliteDatasource";
import { User } from "../entities/user/User";
import { CreateLinkDto } from "../entities/link/CreateLinkDTO";
import { Link } from "../entities/link/Link";
import { DataSource, Repository } from "typeorm";

@Injectable()
export class LinkService {
  private linkRepository: Repository<Link>;

  constructor(@Inject(SqliteDatasource) private dataSource: DataSource) {
    this.linkRepository = this.dataSource.getRepository(Link);
  }

  async createLink(linkData: CreateLinkDto, user: User): Promise<Link> {
    const link = this.linkRepository.create({
      ...linkData,
      user
    });
    return this.linkRepository.save(link);
  }

  async getLinksForUser(user: User): Promise<Link[]> {
    return this.linkRepository.find({
      where: { user: { id: user.id } },
      relations: ["user"],
      select: ["id", "service", "serviceIdentifier", "text"]
    });
  }

  async getLinkById(id: string): Promise<Link | null> {
    const link = await this.linkRepository.findOne({
      where: { id },
      relations: ["user"],
      select: ["id", "service", "serviceIdentifier"]
    });

    if (!link) {
      throw new NotFound("Link not found");
    }

    return link;
  }

  async getLinkContentById(id: string, user: User): Promise<string> {
    const link = await this.linkRepository.findOne({
      where: { id },
      relations: ["user"]
    });

    if (!link) {
      throw new NotFound("Link not found");
    }

    if (link.user.serviceIdentifier !== user.serviceIdentifier) {
      throw new Forbidden("You are not authorized to view this link content");
    }

    return link.text;
  }
}
